/* Autor: Sergio Blanco Cuaresma
 * Data: Agost 2002
 * Descripci: TAD Hashing -> crea taules de hashing de tamany determinat
 *             Implementacio: Taula encadenada indirecte
 */

public class Hashing {
  private NodeHash[] taula;
  private int tamany;

  Hashing(int size){
    tamany = size;
    taula = new NodeHash[tamany];
    // Inicialitzem taula
    for (int i=0;i<tamany;i++){
      taula[i] = new NodeHash("", null);
    }
  }

  // Funci que transforma un String en un integer, el qual ser utilitzat
  // per trobar una posci. El clcul es realitza passant cada lletra al seu
  // valor ASCII, sumant-los i finalment ens assegurem que no soprepassa el
  // tamany de la taula fent un mdul.
  private int hash(String s){
    int posicio=0;

    for (int i=0; i<s.length();i++){
      posicio=posicio+(int)s.charAt(i);
    }
    posicio=posicio%tamany;

    return posicio;
  }
 
  // Mascara que realment crida a una altre funci
  public void afegir(String s, Object o)  throws ElemRepetit{
    this.afegir(s,o,true);
  }
  
  // Afegeix un element a la taula de hashing, si "ambRepetits" esta a true
  // poden haver-hi elements amb l'string repetit, sino no.
  public void afegir(String s, Object o, boolean ambRepetits) throws ElemRepetit {
    int posicio;
    NodeHash tmp;
    
    posicio=hash(s);  // Ens situem en la taula
    if ((taula[posicio]).info()==null){ // Si a aquesta posici mai s'ha ficat res
      (taula[posicio]).info(o);  // Modifiquem el seu contingut
      (taula[posicio]).nom(s);
      return;
    } else if (!ambRepetits) {    // Si es una posici ja ocupada, cas sense repetits
      tmp = taula[posicio];
      if (s.equals(tmp.nom())) {    // Trobem element repetit
        throw new ElemRepetit();
      }
      while (tmp.sinonim()!=null) {   // Busquem l'ltim sinnim
        tmp = tmp.sinonim();
        if (s.equals(tmp.nom())) {    // Trobem element repetit
          throw new ElemRepetit();
        }
      }
      tmp.sinonim(new NodeHash(s, o));
    } else {  // Si es una posici ja ocupada, cas amb repetit
      tmp = taula[posicio];

      while (tmp.sinonim()!=null) { // Busquem l'ultim sinnim
        tmp = tmp.sinonim();
      }
      tmp.sinonim(new NodeHash(s, o));
    }
  }

  // Mscara que realment crida una altra funci
  public Object consultar(String s) throws NoExisteix{
    return this.consultar(s, false);
  }
  
  // Realitza una consulta sobre la taula de Hashing, si "retornarNodeHash" es true
  // retornar tot el NodeHash i d'aquesta forma es podran tractar tots el sinnims o
  // elements repetits des de fora, en cas contrari noms retorna el contingut del
  // NodeHash
  public Object consultar(String s, boolean retornarNodeHash) throws NoExisteix{
    int posicio;
    NodeHash tmp;

    posicio=hash(s);
    tmp=taula[posicio];
    while ((tmp != null) && (!s.equals(tmp.nom()))){  // Busquem
      tmp = tmp.sinonim();
    }
    if (tmp != null){
      if (retornarNodeHash){
        return tmp;
      } else {
        return tmp.info();
      }
    } else {
      throw new NoExisteix();
    }
    
  }

  
}
